@@ -23,15 +23,8 @@ Usage:
   python trunk/wire.py > wiki/JsonWireProtocol.wiki
 """
 
-class ErrorCode(object):
-  def __init__(self, code, summary, detail):
-    self.code = code
-    self.summary = summary
-    self.detail = detail
-
-  def ToWikiTableString(self):
-    return "|| %d || `%s` || %s ||" % (self.code, self.summary, self.detail)
-
+import re
+import sys
 
 class Resource(object):
   def __init__(self, path):
@@ -205,9 +198,157 @@ class Method(object):
         self.summary[:self.summary.find('.') + 1].replace('\n', '').strip())
         
 
+class ErrorCode(object):
+  def __init__(self, code, summary, detail):
+    self.code = code
+    self.summary = summary
+    self.detail = detail
+
+  def ToWikiTableString(self):
+    return '|| %d || `%s` || %s ||' % (self.code, self.summary, self.detail)
+
+def log(string):
+  sys.stderr.write(str(string) + '\n')
+
+class AbstractErrorCodeGatherer(object):
+  def __init__(self, name, path_to_error_codes, regex):
+    self.name = name
+    self.path_to_error_codes = path_to_error_codes
+    self.regex = regex
+
+  def __str__(self):
+    return self.name
+
+  def get_error_codes(self):
+    error_codes = {}
+    error_codes_file = open(self.path_to_error_codes, 'r')
+    try:
+      for line in error_codes_file:
+        match = self.regex.match(line)
+        if match is not None:
+          name, code = self.extract_from_match(match)
+          error_codes[code] = name
+    finally:
+      error_codes_file.close()
+    return error_codes
+
+  def extract_from_match(self, match):
+    raise NotImplementedError
+
+class JavaErrorCodeGatherer(AbstractErrorCodeGatherer):
+  def __init__(self, path_to_error_codes):
+    super(JavaErrorCodeGatherer, self).__init__( \
+      'Java',
+      path_to_error_codes, \
+      re.compile('^\s*public static final int ([A-Z_]+) = (\d+);$'))
+
+  def extract_from_match(self, match):
+    return match.group(1), int(match.group(2))
+
+class JavascriptErrorCodeGatherer(AbstractErrorCodeGatherer):
+  def __init__(self, path_to_error_codes, name):
+    super(JavascriptErrorCodeGatherer, self).__init__( \
+      name,
+      path_to_error_codes, \
+      re.compile('^\s*([A-Z_]+): (\d+)'))
+
+  def extract_from_match(self, match):
+    return match.group(1), int(match.group(2))
+
+class RubyErrorCodeGatherer(AbstractErrorCodeGatherer):
+  def __init__(self, path_to_error_codes):
+    super(RubyErrorCodeGatherer, self).__init__( \
+      'Ruby',
+      path_to_error_codes, \
+      re.compile('^\s*(([A-Z][a-z]*)+),?\s*# (\d+)$'))
+
+  def extract_from_match(self, match):
+    return match.group(1), int(match.group(len(match.groups())))
+
+class PythonErrorCodeGatherer(AbstractErrorCodeGatherer):
+  def __init__(self, path_to_error_codes):
+    super(PythonErrorCodeGatherer, self).__init__( \
+      'Python',
+      path_to_error_codes, \
+      re.compile('^\s*([A-Z_]+) = (\d+)$'))
+
+  def extract_from_match(self, match):
+    return match.group(1), int(match.group(2))
+
+class CErrorCodeGatherer(AbstractErrorCodeGatherer):
+  def __init__(self, path_to_error_codes):
+    super(CErrorCodeGatherer, self).__init__( \
+      'C',
+      path_to_error_codes, \
+      re.compile('^#define ([A-Z]+)\s+(\d+)$'))
+
+  def extract_from_match(self, match):
+    return match.group(1), int(match.group(2))
+
+class CSharpErrorCodeGatherer(AbstractErrorCodeGatherer):
+  def __init__(self, path_to_error_codes):
+    super(CSharpErrorCodeGatherer, self).__init__( \
+      'C#',
+      path_to_error_codes, \
+      re.compile('^\s*(([A-Z][a-z]*)+) = (\d+)'))
+
+  def extract_from_match(self, match):
+    return match.group(1), int(match.group(len(match.groups())))
+
+class ErrorCodeChecker(object):
+  def __init__(self):
+    self.gatherers = []
+    self.inconsistencies = {}
+
+  def using(self, gatherer):
+    self.gatherers.append(gatherer)
+    return self
+
+  def check_error_codes_are_consistent(self, json_error_codes):
+    log('Checking error codes are consistent across languages and \
+browsers')
+    for gatherer in self.gatherers:
+      self.compare(gatherer, json_error_codes)
+    if not self.inconsistencies:
+      log('Error codes are consistent')
+    for code,(present,missing) in self.inconsistencies.items():
+      log('Error code %d was present in %s but not %s' % (code, present, missing))
+
+  def add_inconsistency(self, code, present_in, missing_from):
+    if self.inconsistencies.has_key(code):
+      already_present, already_missing = self.inconsistencies[code]
+      already_present.add(present_in)
+      already_missing.add(missing_from)
+    else:
+      self.inconsistencies[code] = (set([present_in]), set([missing_from]))
+
+  def compare(self, gatherer, raw_json_error_codes):
+    log('Checking %s (%s)' % (gatherer, gatherer.path_to_error_codes))
+    gathered_error_codes = gatherer.get_error_codes()
+    json_error_codes = map(lambda code: code.code, raw_json_error_codes)
+    for json_error_code in json_error_codes:
+      if not gathered_error_codes.has_key(json_error_code):
+        self.add_inconsistency(json_error_code, 'JSON', str(gatherer))
+    for gathered_code,_ in gathered_error_codes.items():
+      if not gathered_code in json_error_codes:
+        self.add_inconsistency(gathered_code, str(gatherer), 'JSON')
+      
+
 def main():
   error_codes = [
       ErrorCode(0, 'Success', 'The command executed successfully.'),
+#      ErrorCode(1, 'IndexOutOfBounds', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
+#      ErrorCode(2, 'NoCollection', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
+#      ErrorCode(3, 'NoString', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
+#      ErrorCode(4, 'NoStringLength', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
+#      ErrorCode(5, 'NoStringWrapper', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
+#      ErrorCode(6, 'NoSuchDriver', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
       ErrorCode(7, 'NoSuchElement', 'An element could not be located on the \
 page using the given search parameters.'),
       ErrorCode(8, 'NoSuchFrame', 'A request to switch to a frame could not be \
@@ -224,21 +365,54 @@ completed because the element is in an invalid state (e.g. attempting to \
 click a disabled element).'),
       ErrorCode(13, 'UnknownError', 'An unknown server-side error occurred \
 while processing the command.'),
+#      ErrorCode(14, 'ExpectedError', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
       ErrorCode(15, 'ElementIsNotSelectable', 'An attempt was made to select \
 an element that cannot be selected.'),
+#      ErrorCode(16, 'NoSuchDocument', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
       ErrorCode(17, 'JavaScriptError', 'An error occurred while executing user \
 supplied !JavaScript.'),
+#      ErrorCode(18, 'NoScriptResult', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
       ErrorCode(19, 'XPathLookupError', 'An error occurred while searching for \
 an element by XPath.'),
+#      ErrorCode(20, 'NoSuchCollection', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
+      ErrorCode(21, 'Timeout', 'An operation did not complete before its \
+timeout expired.'),
+#      ErrorCode(22, 'NullPointer', 'This is probably an unused \
+#implementation detail of an old version of the IEDriver.'),
       ErrorCode(23, 'NoSuchWindow', 'A request to switch to a different window \
 could not be satisfied because the window could not be found.'),
       ErrorCode(24, 'InvalidCookieDomain', 'An illegal attempt was made to set \
 a cookie under a different domain than the current page.'),
       ErrorCode(25, 'UnableToSetCookie', 'A request to set a cookie\'s value \
 could not be satisfied.'),
-      ErrorCode(28, 'Timeout', 'A command did not complete before its timeout \
-expired.')
+      ErrorCode(26, 'UnexpectedAlertOpen', 'A modal dialog was open, blocking \
+this operation'),
+      ErrorCode(27, 'NoAlertOpenError', 'An attempt was made to operate on a \
+modal dialog when one was not open.'),
+      ErrorCode(28, 'ScriptTimeout', 'A script did not complete before its \
+timeout expired.'),
+      ErrorCode(29, 'InvalidElementCoordinates', 'The coordinates provided to \
+an interactions operation are invalid.'),
+      ErrorCode(30, 'IMENotAvailable', 'IME was not available.'),
+      ErrorCode(31, 'IMEEngineActivationFailed', 'An IME engine could not be \
+started.'),
+      ErrorCode(32, 'InvalidSelector', 'Argument was an invalid selector \
+(e.g. XPath/CSS).')
   ]
+  
+  ErrorCodeChecker() \
+  .using(JavaErrorCodeGatherer('java/client/src/org/openqa/selenium/remote/ErrorCodes.java')) \
+  .using(JavascriptErrorCodeGatherer('javascript/atoms/error.js', 'Javascript atoms')) \
+  .using(JavascriptErrorCodeGatherer('javascript/firefox-driver/js/errorcode.js', 'Javascript firefox driver')) \
+  .using(RubyErrorCodeGatherer('rb/lib/selenium/webdriver/common/error.rb')) \
+  .using(PythonErrorCodeGatherer('py/selenium/webdriver/remote/errorhandler.py')) \
+  .using(CErrorCodeGatherer('cpp/webdriver-interactions/errorcodes.h')) \
+  .using(CSharpErrorCodeGatherer('dotnet/src/WebDriver/WebDriverResult.cs')) \
+  .check_error_codes_are_consistent(error_codes)
 
   resources = []
  
